//
//  ViewController.swift
//  RFID
//
//  Created by Phidgets on 2018-02-01.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController, UITextFieldDelegate {

    
    let ch = RFID()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    
    @IBOutlet var stackView: UIStackView!
    @IBOutlet var tagStringLabel: UILabel!
    @IBOutlet var protocolLabel: UILabel!
    @IBOutlet var antennaEnabledSwitch: UISwitch!
    
    @IBOutlet var writeStackView: UIStackView!
    @IBOutlet var tagStringText: UITextField!
    @IBOutlet var protocolSegment: UISegmentedControl!
    @IBOutlet var lockSwitch: UISwitch!
    
    @IBOutlet var topConstraint: NSLayoutConstraint!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.tag.addHandler(tag_handler)
            let _ = ch.tagLost.addHandler(taglost_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            let attachedDevice = sender as! RFID
            let antennaEnabled = try attachedDevice.getAntennaEnabled()
            let deviceID = try attachedDevice.getDeviceID()
            DispatchQueue.main.async {
                self.antennaEnabledSwitch.isOn = antennaEnabled
                self.stackView.isHidden = false
                if(deviceID == DeviceID.PN_1023){
                    self.writeStackView.isHidden = true
                    self.topConstraint.constant = 10
                }
                else if (deviceID == DeviceID.PN_1024){
                    self.writeStackView.isHidden = false
                    self.topConstraint.constant = 300
                }
            }

        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    func tag_handler(sender: RFID, data: (tag: String, proto: RFIDProtocol)){
        DispatchQueue.main.async {
            self.tagStringLabel.text = data.tag
            self.protocolLabel.text = (data.proto == RFIDProtocol.EM4100) ? "EM4100" : (data.proto == RFIDProtocol.ISO11785_FDX_B) ? "ISO11785_FDX_B" : "phidgetTag"
        }
    }
    
    func taglost_handler(sender: RFID, data: (tag: String, proto: RFIDProtocol)){
        DispatchQueue.main.async {
            self.tagStringLabel.text = ""
            self.protocolLabel.text = ""
        }
    }
    
    //GUI controls
    @IBAction func enableAntenna(_ sender: Any) {
        let swtch = sender as! UISwitch
        do{
            try ch.setAntennaEnabled(swtch.isOn)
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func writeTag(_ sender: Any) {
        var lock = false
        if(lockSwitch.isOn){
            let alert = UIAlertController(title: "WARNING", message: "This action will lock the RFID tag and prevent further writes.\n\nAre you sure you want to continue?", preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
                lock = true
            }))
            alert.addAction(UIAlertAction(title: "Cancel", style: .default, handler: {action -> Void in
                return
            }))
            self.present(alert, animated: true, completion: nil)
            
        }
        do{
            let tagString = tagStringText.text!
            let proto = (protocolSegment.selectedSegmentIndex == 0) ? RFIDProtocol.EM4100 : (protocolSegment.selectedSegmentIndex == 1) ? RFIDProtocol.ISO11785_FDX_B : RFIDProtocol.phidgetTAG
            try ch.write(tagString: tagString, proto: proto, lockTag: lock)            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    

    //MARK: navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
    }
    
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    //MARK: Text field
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        self.view.endEditing(true)
        return false
    }
}

